# -*- coding: utf-8 -*-

"""
verizon

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from verizon.api_helper import APIHelper
from verizon.configuration import Server
from verizon.http.api_response import ApiResponse
from verizon.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from verizon.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from apimatic_core.authentication.multiple.or_auth_group import Or
from verizon.models.m_5g_bidevice_detailsresponse import M5gBideviceDetailsresponse
from verizon.models.m_5g_bi_request_response import M5gBiRequestResponse
from verizon.exceptions.m_5g_bi_rest_error_response_exception import M5gBiRestErrorResponseException


class M5gBIDeviceActionsController(BaseController):

    """A Controller to access Endpoints in the verizon API."""
    def __init__(self, config):
        super(M5gBIDeviceActionsController, self).__init__(config)

    def business_internetlist_device_information(self,
                                                 body):
        """Does a POST request to /actions/list.

        Uses the decive's Integrated Circuit Card Identification Number
        (ICCID) to retrive and display the device's properties.

        Args:
            body (M5gBideviceId): Device Profile Query

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. The
                device's details will be returned from a successful request.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.THINGSPACE)
            .path('/actions/list')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Or(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(M5gBideviceDetailsresponse.from_dictionary)
            .is_api_response(True)
            .local_error('default', 'Error response', M5gBiRestErrorResponseException)
        ).execute()

    def business_internetactivate_using_post(self,
                                             body):
        """Does a POST request to /actions/activate.

        Uses the device's ICCID and IMEI to activate service.

        Args:
            body (M5gBiactivateRequest): Activate 5G BI service. Defining
                <code>publicIpRestriction</code> as "Unrestricted" or
                "Restricted" is required for activating as Public Static.
                Leave  <code>publicIpRestriction</code> undefined to activate
                as Public Dynamic. Removing <code>publicIpRestriction</code>
                from the request will activate as Mobile Private Network (MPN).

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A request
                ID is returned as a successful response. Use a callback to see
                the details associated with the request ID.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.THINGSPACE)
            .path('/actions/activate')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Or(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(M5gBiRequestResponse.from_dictionary)
            .is_api_response(True)
            .local_error('default', 'Error response', M5gBiRestErrorResponseException)
        ).execute()

    def business_internet_serviceplanchange(self,
                                            body):
        """Does a PUT request to /actions/plan.

        Change a device's service plan to use 5G BI.

        Args:
            body (M5gBichangeRequest): This endpoint is for use when changing
                a device's service plan to a 5G BI service plan. The service
                plan can change for an active device up to four times per
                month but will require address validation for each change. The
                service plan cannot be changed for a device while its service
                is suspended.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A request
                ID is returned as a successful response. Use a callback to see
                the details associated with the request ID.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.THINGSPACE)
            .path('/actions/plan')
            .http_method(HttpMethodEnum.PUT)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Or(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(M5gBiRequestResponse.from_dictionary)
            .is_api_response(True)
            .local_error('default', 'Error response', M5gBiRestErrorResponseException)
        ).execute()
